#include <stdio.h>
#include <stdlib.h>
#include <cal.h>
#include <calcl.h>

#include "cal-utils.h"

void fatal(const char *func_name)
{
    const char *cal_msg = calGetErrorString();
    const char *comp_msg = calclGetErrorString();
    fprintf(stderr, "%s failed\ncal error: %s\ncalcl error: %s\n",
	    func_name, cal_msg, comp_msg);
    // calGetErrorString error messages are prematurely truncated because of
    // stray NUL chars. print the message until 3 consecutive NUL chars are
    // encountered
    fprintf(stderr, "Full CAL error: ");
    for (int i = 0; i < 128; i++) {
	if (i > 1 && !cal_msg[i - 2] && !cal_msg[i - 1] && !cal_msg[i])
	    break;
	if (cal_msg[i])
	    fprintf(stderr, "%c", cal_msg[i]);
    }
    fprintf(stderr, "\n");
    exit(1);
}

void show_ver(void)
{
    CALuint major, minor, imp;
    if (CAL_RESULT_OK != calGetVersion(&major, &minor, &imp))
	fatal("calGetVersion");
    printf("CAL version %u.%u.%u\n", major, minor, imp);
}

const char *target_name(CALtarget target, CALuint revision)
{
    static char _out[128];
    char *name;
    switch (target) {
	case CAL_TARGET_600: name = "R600 GPU"; break;
	case CAL_TARGET_610: name = "RV610 GPU"; break;
	case CAL_TARGET_630: name = "RV630 GPU"; break;
	case CAL_TARGET_670: name = "RV670 GPU"; break;
	case CAL_TARGET_7XX: name = "R700 class GPU"; break;
	case CAL_TARGET_770: name = "RV770 GPU"; break;
	case CAL_TARGET_710: name = "RV710 GPU"; break;
	case CAL_TARGET_730: name = "RV730 GPU"; break;
	case CAL_TARGET_CYPRESS: name = "Cypress GPU"; break;
	case CAL_TARGET_JUNIPER: name = "Juniper GPU"; break;
	case CAL_TARGET_REDWOOD: name = "Redwood GPU"; break;
	case CAL_TARGET_CEDAR: name = "Cedar GPU"; break;
	default: name = "unknown"; break;
    }
    sprintf(_out, "%s (tgt.rev %u.%u)", name, target, revision);
    return _out;
}

void display_attribs(CALdeviceattribs *a)
{
    printf(
	    "target                %u\n"
	    "localRAM              %u MB\n"
	    "uncachedRemoteRAM     %u MB\n"
	    "cachedRemoteRAM       %u MB\n"
	    "engineClock           %u MHz\n"
	    "memoryClock           %u MHz\n"
	    "wavefrontSize         %u\n"
	    "numberOfSIMD          %u\n"
	    "doublePrecision       %u\n"
	    "localDataShare        %u\n"
	    "globalDataShare       %u\n"
	    "globalGPR             %u\n"
	    "computeShader         %u\n"
	    "memExport             %u\n"
	    "pitch_alignment       %u\n"
	    "surface_alignment     %u\n"
	    "numberOfUAVs          %u\n"
	    "bUAVMemExport         %u\n"
	    "b3dProgramGrid        %u\n"
	    "numberOfShaderEngines %u\n"
	    "targetRevision        %u\n"
	    , a->target, a->localRAM, a->uncachedRemoteRAM, a->cachedRemoteRAM,
	    a->engineClock, a->memoryClock, a->wavefrontSize, a->numberOfSIMD,
	    a->doublePrecision, a->localDataShare, a->globalDataShare,
	    a->globalGPR, a->computeShader, a->memExport, a->pitch_alignment,
	    a->surface_alignment, a->numberOfUAVs, a->bUAVMemExport,
	    a->b3dProgramGrid, a->numberOfShaderEngines, a->targetRevision);
}
