#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <cal.h>
#include <calcl.h>

#include "cpu-md5.h"

/*
 * MD5 steps reversing.
 */

#define I(x, y, z)          ((y) ^ ((x) | ~(z)))
#define REV_STEP(v1, v2, v3, v4, x, t, s) \
    do { \
	(v1) -= (v2); \
	(v1) = (((v1) & 0xffffffff) >> (s)) | ((v1) << (32 - (s))); \
	(v1) -= I((v2), (v3), (v4)) + (x) + (t); \
    } while (0)

void md5_reverse(uint32_t *x, uint32_t *hash, uint32_t *d_res)
{
    uint32_t a, b, c, d;
    a = hash[0];
    b = hash[1];
    c = hash[2];
    d = hash[3];
    a -= 0x67452301;
    b -= 0xefcdab89;
    c -= 0x98badcfe;
    d -= 0x10325476;
    /* round 4, in reverse */
    REV_STEP(b, c, d, a, x[9], 0xeb86d391, 21); // step 63
    REV_STEP(c, d, a, b, x[2], 0x2ad7d2bb, 15);
    REV_STEP(d, a, b, c, x[11], 0xbd3af235, 10);
    REV_STEP(a, b, c, d, x[4], 0xf7537e82, 6);
    REV_STEP(b, c, d, a, x[13], 0x4e0811a1, 21);
    REV_STEP(c, d, a, b, x[6], 0xa3014314, 15);
    REV_STEP(d, a, b, c, x[15], 0xfe2ce6e0, 10);
    REV_STEP(a, b, c, d, x[8], 0x6fa87e4f, 6);
    REV_STEP(b, c, d, a, x[1], 0x85845dd1, 21);
    REV_STEP(c, d, a, b, x[10], 0xffeff47d, 15);
    REV_STEP(d, a, b, c, x[3], 0x8f0ccc92, 10);
    REV_STEP(a, b, c, d, x[12], 0x655b59c3, 6);
    REV_STEP(b, c, d, a, x[5], 0xfc93a039, 21);
    REV_STEP(c, d, a, b, x[14], 0xab9423a7, 15);
    REV_STEP(d, a, b, c, x[7], 0x432aff97, 10); // step 49
    //REV_STEP(a, b, c, d, x[0], 0xf4292244, 6); // step 48
    *d_res = d;
}

void usage(const char *name)
{
    fprintf(stdout, "Usage: %s [-h]\n"
	    "  -h        Print this help\n"
	    , name);
}

int main(int argc, char** argv)
{
    int opt;
    while ((opt = getopt(argc, argv, "h")) != -1) {
	switch (opt) {
	    case 'h':
		usage(argv[0]);
		exit(0);
	    default:
		usage(argv[0]);
		exit(1);
	}
    }
    if (optind < argc)
      {
	fprintf(stderr, "Extra option: %s\n", argv[optind]);
	usage(argv[0]);
	exit(1);
      }
    MD5_CTX ctx;
    unsigned char res[16];
    MD5_Init(&ctx);
    MD5_Final(res, &ctx);
    for (unsigned i = 0; i < sizeof (res); i++)
	printf("%02x", res[i]);
    printf("\n");
}
